/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/*
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 2001 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s):
 *   Darin Fisher <darin@netscape.com> (original author)
 */

#include "nsEventSender.h"
#include "nsIEventQueueService.h"
#include "nsIServiceManager.h"

static NS_DEFINE_CID(kEventQueueService, NS_EVENTQUEUESERVICE_CID);

//-----------------------------------------------------------------------------
// nsEventSender implementation
//-----------------------------------------------------------------------------

nsresult
nsEventSender::FireEvent(nsAEvent *event)
{
    NS_ENSURE_TRUE(mEventQ, NS_ERROR_NOT_INITIALIZED);

    PRStatus status = mEventQ->PostEvent(event->GetPLEvent());
    return status == PR_SUCCESS ? NS_OK : NS_ERROR_FAILURE;
}

nsresult
nsEventSender::SetEventQueue(nsIEventQueue *eq)
{
    nsresult rv = NS_OK;
    if ((eq == NS_CURRENT_EVENTQ) || (eq == NS_UI_THREAD_EVENTQ)) {
        nsCOMPtr<nsIEventQueueService> serv = do_GetService(kEventQueueService, &rv);
        if (NS_FAILED(rv)) return rv;
        rv = serv->GetSpecialEventQueue((PRInt32) eq, getter_AddRefs(mEventQ));
    }
    else
        mEventQ = eq;
    return rv;
}

//-----------------------------------------------------------------------------
// nsAEvent implementation
//-----------------------------------------------------------------------------

nsAEvent::nsAEvent()
{
    PL_InitEvent(&mEvent, nsnull,
        (PLHandleEventProc) nsAEvent::HandlePLEvent,
        (PLDestroyEventProc) nsAEvent::DestroyPLEvent);
}

void PR_CALLBACK
nsAEvent::HandlePLEvent(PLEvent *plev)
{
    nsAEvent *ev = NS_REINTERPRET_CAST(nsAEvent *, plev);
    NS_ASSERTION(ev, "null event");

    // Pass control to the real event handler
    if (ev)
        ev->HandleEvent();
}

void PR_CALLBACK
nsAEvent::DestroyPLEvent(PLEvent *plev)
{
    nsAEvent *ev = NS_REINTERPRET_CAST(nsAEvent *, plev);
    NS_ASSERTION(ev, "null event");
    delete ev;
}
